@@ -4,8 +4,9 @@ from coopstructs.vectors import Vector2
 import math
 from typing import List, Callable, Dict
 from coopstructs.toggles import EnumToggleable
-from coopstructs.geometry import Rectangle, Line
+from coopstructs.geometry import Rectangle, Line, Triangle, Circle
 import numpy as np
+from uuid import uuid4
 
 class Orientation(Enum):
     UP_LEFT = 3
@@ -92,6 +93,10 @@ class Arc(Curve):
     @classmethod
     def from_points(cls, points: List[Vector2], naming_provider: Callable[[], str]) -> (List, List):
         curves = []
+
+        if len(points) == 2 and points[0] == points[1]:
+            return curves, points
+
         for ii in range(len(points) - 1):
             orientation = cls.orientation(points[ii], points[ii + 1])[0]
             if orientation in (Orientation.UP, Orientation.RIGHT, Orientation.DOWN, Orientation.LEFT):
@@ -252,6 +257,124 @@ class Arc(Curve):
         else:
             return next_point
 
+class CircularArc(Curve):
+    @classmethod
+    def from_points(cls, points: List[Vector2], naming_provider: Callable[[], str]) -> (List, List):
+        leftover_points = []
+        leftover_count = (len(points) - 1) % 3 if len(points) >= 3 else len(points)
+        leftover_points = points[-leftover_count:] if leftover_count != 0 else leftover_points
+        used_points = points[:-leftover_count] if leftover_count != 0 else points
+
+        curves = []
+
+        if len(points) < 3:
+            return curves, points
+
+        # A Cubic Bezier requires 4 control points, so operate in blocks of 4 (re-using the last each time)
+        for ii in range(0, len(used_points) - 3, 3):
+            curves.append(CircularArc(naming_provider(),
+                                      origin=points[ii],
+                                      destination=points[ii+1],
+                                      center=points[ii+2]))
+
+        return curves, leftover_points
+
+    @classmethod
+    def from_3_consecutive_points(cls, a: Vector2, b: Vector2, c: Vector2, naming_provider: Callable[[], str]):
+
+        circle = Circle.from_boundary_points(a, b, c)
+
+        rads_ab = circle.rads_between_points(a, b)
+        rads_ac = circle.rads_between_points(a, c)
+
+        if rads_ac > rads_ab:
+            ret = CircularArc(id=naming_provider(),
+                              origin=a,
+                              destination=c,
+                              center=circle.center)
+        else:
+            ret = CircularArc(id=naming_provider(),
+                              origin=c,
+                              destination=a,
+                              center=circle.center)
+
+        return ret
+
+
+
+    def __init__(self, id: str, origin:Vector2, destination:Vector2, center: Vector2):
+        """ Create a circle given a center, start, and end. The arc is assumed to be the counter-clockwise movement from origin to
+        destination, so enter the o and d appropriately based on which segment between points is required"""
+
+        if origin == center or destination == center or origin == destination:
+            raise ValueError(f"Origin, destination and center must all be different. Provided:"
+                             f"\no: {origin}"
+                             f"\nd: {destination}"
+                             f"\nc: {center}")
+
+        centered_o = origin - center
+        centered_d = destination - center
+        min_length = min(centered_o.length(), centered_d.length())
+
+        scaled_o = centered_o.scaled_to_length(min_length)
+        scaled_d = centered_d.scaled_to_length(min_length)
+
+        used_o = center + scaled_o
+        used_d = center + scaled_d
+
+        super().__init__(id, origin=used_o)
+        self.circle = Circle(center, radius=min_length)
+        self.destination = used_d
+
+    @property
+    def center(self):
+        return self.circle.center
+
+    @property
+    def angle_rads(self):
+        return self.circle.rads_between_points(self.origin, self.destination)
+
+    @property
+    def arc_length(self):
+        return self.radius * self.angle_rads
+
+    @property
+    def radius(self):
+        return self.circle.radius
+
+    @property
+    def origin_rads(self):
+        return self.circle.rads_of_point(self.origin)
+
+    @property
+    def destination_rads(self):
+        return self.circle.rads_of_point(self.destination)
+
+
+    def _end_point(self):
+        return self.destination
+
+    def _control_points(self) -> List[Vector2]:
+        return [self.origin, self.destination, self.center]
+
+    def line_representation(self) -> List[Line]:
+        pass
+
+    def force_continuity(self, next_point: Vector2) -> Vector2:
+        pass
+
+    def compute_circulararc_points(self, numPoints: int=None):
+        if numPoints is None:
+            numPoints = 30
+
+        step_size = self.angle_rads / numPoints
+        start = self.circle.rads_of_point(self.origin)
+        points = []
+        for ii in range(0, numPoints):
+            point = self.circle.point_at_angle(radians=start + ii * step_size)
+            points.append(point)
+
+        return points
 
 
 class LineCurve(Curve):
@@ -277,12 +400,43 @@ class LineCurve(Curve):
         return self.destination
 
     def length(self):
-        return Line(self.origin, self.destination).length()
+        return Line(self.origin, self.destination).length
 
     def force_continuity(self, next_point: Vector2) -> Vector2:
         return next_point
 
 class CubicBezier(Curve):
+    @classmethod
+    def as_circular_arcs(cls, curve, tolerance:float = 10):
+        if not type(curve) == CubicBezier:
+            raise TypeError(f"provided value was not of type {CubicBezier}. {type(curve)} provided")
+
+        # find inflection points
+        inflection_points = curve.inflection_points()
+
+        # handle inflection points on curve
+        if len(inflection_points) > 0:
+            sub_divisions = curve.sub_divide_at_t(inflection_points[0][1])
+            arcs = [CubicBezier.as_circular_arcs(x, tolerance) for x in sub_divisions]
+            return [item for sublist in arcs for item in sublist]
+
+        # handle check if circle is a good approximater
+        if curve.origin != curve.end_point:
+            circle, deviation = curve.biarc_circle()
+            if circle is None:
+                pass
+            elif deviation < tolerance:
+                return [CircularArc.from_3_consecutive_points(a=circle.known_boundary_points[0],
+                                                              b=circle.known_boundary_points[1],
+                                                              c=circle.known_boundary_points[2],
+                                                              naming_provider=lambda : str(uuid4()))]
+
+        # handle subdivide and try again
+        sub_divisions = curve.sub_divide_at_t(0.5)
+        arcs = [CubicBezier.as_circular_arcs(x, tolerance) for x in sub_divisions]
+        return [item for sublist in arcs for item in sublist]
+
+
     @classmethod
     def from_points(cls, points: List[Vector2], naming_provider: Callable [[], str]) -> (List, List):
         leftover_points = []
@@ -302,9 +456,15 @@ class CubicBezier(Curve):
         return curves, leftover_points
 
     def __init__(self, id:str, control_points: List[Vector2]):
+
+        if all(point.distance_from(control_points[0]) < .001 for point in control_points):
+            raise ValueError(f"the provided control points are invalid as they are all the same: {control_points[0]}")
+
         Curve.__init__(self, id, control_points[0])
         self._cps = control_points
 
+
+
     def line_representation(self):
         return self.compute_curve_lines()
 
@@ -394,11 +554,145 @@ class CubicBezier(Curve):
     def force_continuity(self, next_point: Vector2) -> Vector2:
         return next_point.project_onto(self.control_points[-1], self.control_points[-2])
 
-# class EasyInputCubicBezier(CubicBezier):
-#     def __init__(self, id:str, control_points: List[Vector2]):
-#         super().__init__()
 
 
+    def biarc_circle(self) -> (Circle, float):
+        p1 = self.control_points[0]
+        p4 = self.control_points[3]
+
+        p1_ = p1 + self.derivative_at_t(0)
+        p4_ = p4 + self.derivative_at_t(1)
+
+        line1 = Line(origin=p1, destination=p1_)
+        line2 = Line(origin=p4, destination=p4_)
+
+        try:
+            V = line1.intersection(line2)
+        except:
+            return None, None
+
+        tri = Triangle(p1, V, p4)
+        incentre = tri.incentre()
+
+        biarc_circ = Circle.from_boundary_points(p1, incentre, p4)
+
+        max_delta = 0
+        for point in self.compute_bezier_points([(x.x, x.y) for x in self.control_points], 10):
+            point = Vector2(point[0], point[1])
+            dist_from_center = (point - biarc_circ.center).length()
+            deviation = abs(biarc_circ.radius - dist_from_center)
+            if deviation > max_delta:
+                max_delta = deviation
+
+        return biarc_circ, max_delta
+
+
+
+
+
+    def sub_divide_at_t(self, t: float):
+        if not (0 < t < 1):
+            raise ValueError(f"t must be between 0 and 1 but {t} was provided")
+
+        p1 = self.control_points[0]
+        p2 = self.control_points[1]
+        p3 = self.control_points[2]
+        p4 = self.control_points[3]
+        r2 = p1 + t * (p2 - p1)
+        s3 = p3 + t * (p4 - p3)
+        M = (p2 + t * (p3 - p2))
+        r3 = r2 + t * (M - r2)
+        s2 = M + t * (s3 - M)
+
+        point_t = self.point_at_t(t)
+        return [CubicBezier(id=str(uuid4()), control_points=[p1, r2, r3, point_t]),
+                CubicBezier(id=str(uuid4()), control_points=[point_t, s2, s3, p4])]
+
+
+
+    def point_at_t(self, t: float):
+        if not (0 < t < 1):
+            raise ValueError(f"t must be between 0 and 1 but {t} was provided")
+
+        b0x = self.control_points[0].x
+        b0y = self.control_points[0].y
+        b1x = self.control_points[1].x
+        b1y = self.control_points[1].y
+        b2x = self.control_points[2].x
+        b2y = self.control_points[2].y
+        b3x = self.control_points[3].x
+        b3y = self.control_points[3].y
+
+        f_x = lambda t: b0x * (1 - t) ** 3 + 3 * b1x * (1 - t) ** 2 * t + 3 * b2x * (1 - t) * t ** 2 + b3x * t ** 3
+        f_y = lambda t: b0y * (1 - t) ** 3 + 3 * b1y * (1 - t) ** 2 * t + 3 * b2y * (1 - t) * t ** 2 + b3y * t ** 3
+
+        return Vector2(f_x(t), f_y(t))
+
+    def derivative_at_t(self, t: float):
+        if not (0 <= t <= 1):
+            raise ValueError(f"t must be between 0 and 1 but {t} was provided")
+
+        b0x = self.control_points[0].x
+        b0y = self.control_points[0].y
+        b1x = self.control_points[1].x
+        b1y = self.control_points[1].y
+        b2x = self.control_points[2].x
+        b2y = self.control_points[2].y
+        b3x = self.control_points[3].x
+        b3y = self.control_points[3].y
+
+        a_x = 3 * (b1x - b0x)
+        b_x = 3 * (b2x - b1x)
+        c_x = 3 * (b3x - b2x)
+
+        a_y = 3 * (b1y - b0y)
+        b_y = 3 * (b2y - b1y)
+        c_y = 3 * (b3y - b2y)
+
+        f_1_x = lambda t: a_x * (1 - t) ** 2 + 2 * b_x * (1 - t) * t + c_x * t ** 2
+        f_1_y = lambda t: a_y * (1 - t) ** 2 + 2 * b_y * (1 - t) * t + c_y * t ** 2
+
+        ret = Vector2(f_1_x(t), f_1_y(t))
+        return ret
+
+    def inflection_points(self):
+        #https://stackoverflow.com/questions/35901079/calculating-the-inflection-point-of-a-cubic-bezier-curve
+
+        b0x = self.control_points[0].x
+        b0y = self.control_points[0].y
+        b1x = self.control_points[1].x
+        b1y = self.control_points[1].y
+        b2x = self.control_points[2].x
+        b2y = self.control_points[2].y
+        b3x = self.control_points[3].x
+        b3y = self.control_points[3].y
+
+        a = b2x * b1y
+        b = b3x * b1y
+        c = b1x * b2y
+        d = b3x * b2y
+
+        v1 = (-3 * a + 2 * b + 3 * c - d) * 18
+        v2 = (3 * a - b - 3 * c) * 18
+        v3 = (c - a) * 18
+
+        rooter = v2 ** 2 - 4 * v1 * v3
+        if 3 * a + d == 2 * b + 3 * c:
+            return []
+        elif rooter < 0:
+            return []
+
+        sqr = math.sqrt(rooter)
+        e = 2 * v1
+        root1 = (sqr - v2) / e
+        root2 = -(sqr + v2) / e
+
+        valid_root_ts = [x for x in [root1, root2] if 0 < round(x, 1) < 1]
+
+        roots = [(self.point_at_t(t), t) for t in valid_root_ts]
+
+        return roots
+
 
 class CatmullRom(Curve):
 
@@ -546,26 +840,6 @@ class CurveBuilder:
         if any(self.current_points):
             self.current_points.pop()
 
-    # def orientation(self, origin, destination) -> List[Orientation]:
-    #     orientation = None
-    #     if origin.x == destination.x and origin.y > destination.y:
-    #         orientation = [Orientation.UP]
-    #     elif origin.x == destination.x and origin.y < destination.y:
-    #         orientation = [Orientation.DOWN]
-    #     elif origin.x < destination.x and origin.y == destination.y:
-    #         orientation = [Orientation.RIGHT]
-    #     elif origin.x > destination.x and origin.y == destination.y:
-    #         orientation = [Orientation.LEFT]
-    #     elif origin.x > destination.x and origin.y > destination.y:
-    #         orientation = [Orientation.UP_LEFT, Orientation.LEFT_UP]
-    #     elif origin.x > destination.x and origin.y < destination.y:
-    #         orientation = [Orientation.DOWN_LEFT, Orientation.LEFT_DOWN]
-    #     elif origin.x < destination.x and origin.y > destination.y:
-    #         orientation = [Orientation.UP_RIGHT, Orientation.RIGHT_UP]
-    #     elif origin.x < destination.x and origin.y < destination.y:
-    #         orientation = [Orientation.DOWN_RIGHT, Orientation.RIGHT_DOWN]
-    #     return orientation
-
     def curves_from_points(self, curve_type: CurveType, points: List[Vector2]) -> (List[Curve], List[Vector2]):
         # curves = []
         # leftover_points = []
@@ -595,47 +869,6 @@ class CurveBuilder:
 
         return curves, leftover_points
 
-    # def lines_from_points(self, points):
-    #     lines = []
-    #     for ii in range(len(points) - 1):
-    #         lines.append(LineCurve(self.id_provider(), points[ii], points[ii + 1]))
-    #     return lines
-    #
-    # def arcs_from_points(self, points):
-    #     curves = []
-    #     for ii in range(len(points) - 1):
-    #         orientation = self.orientation(points[ii], points[ii + 1])[0]
-    #         if orientation in (Orientation.UP, Orientation.RIGHT, Orientation.DOWN, Orientation.LEFT):
-    #             curves.append(LineCurve(self.id_provider(), points[ii], points[ii + 1]))
-    #         else:
-    #             curves.append(Arc(self.id_provider(), orientation, points[ii],
-    #                         Vector2(abs(points[ii + 1].x - points[ii].x) * 2, abs(points[ii + 1].y - points[ii].y) * 2))
-    #                           )
-    #     return curves
-
-    # def cubicbeziers_from_points(self, points):
-    #     curves = []
-    #
-    #     if len(points) < 4:
-    #         return curves
-    #
-    #     # A Cubic Bezier requires 4 control points, so operate in blocks of 4 (re-using the last each time)
-    #     for ii in range(0, len(points) - 3, 3):
-    #         curves.append(CubicBezier(self.id_provider(), points[ii:ii+4]))
-    #
-    #     # curves.append(CubicBezier(self.id_provider(), points))
-    #     return curves
-
-    # def catmullrom_from_points(self, points):
-    #     curves = []
-    #
-    #     if len(points) < 4:
-    #         return curves
-    #
-    #     # A Cubic Bezier requires 4 control points, so operate in blocks of 3 (re-using the last each time)
-    #     curves.append(CatmullRom(self.id_provider(), points))
-    #     return curves
-
     def curves_from_temp_next_points(self, next_points: List[Vector2]):
         all_points = self.current_points + next_points
         points = []
