@@ -1,4 +1,4 @@
-from typing import List
+from typing import List, Callable, Any
 
 class Toggleable():
     def __init__(self, values: List, starting_index: int = 0):
@@ -6,12 +6,15 @@ class Toggleable():
         self.values = values
         self.index = starting_index
 
-    def toggle(self):
+    def toggle(self, on_toggle_callback: Callable[[Any], Any] = None):
         self.index += 1
 
         if self.index >= len(self.values):
             self.index = 0
 
+        if on_toggle_callback is not None:
+            on_toggle_callback()
+
         return self.value
 
     @property
@@ -19,7 +22,10 @@ class Toggleable():
         return self.values[self.index]
 
     def __repr__(self):
-        return self.value
+        return str(self.value)
+
+    def __eq__(self, other):
+        return self.value == other.value
 
     def copy(self):
         return type(self)(values=self.value, starting_index=self.index)
@@ -44,6 +50,32 @@ class BooleanToggleable(Toggleable):
         default = True if self.index == 1 else False
         return type(self)(default=default)
 
+    def __add__(self, other):
+        return any([self.value, other.value])
+
+    def __gt__(self, other):
+        if self.value and not other.value:
+            return True
+        else:
+            return False
+
+    def __ge__(self, other):
+        if self.value or self.value == other.value == False:
+            return True
+        else:
+            return False
+
+    def __le__(self, other):
+        if other.value or self.value == other.value == False:
+            return True
+        else:
+            return False
+
+    def __lt__(self, other):
+        if not self.value and other.value:
+            return True
+        else:
+            return False
 
 class IntegerRangeToggleable(Toggleable):
     def __init__(self, min: int, max: int, step_size: int = 1):
