@@ -1,8 +1,11 @@
 import math
-from typing import Dict, List
+from typing import Dict, List, Tuple
 import numpy as np
 
 class IVector:
+    def from_ivector(self, ivec):
+        raise NotImplementedError()
+
     def __init__(self, coords: Dict = None):
         if coords is not None:
             self.coords = coords
@@ -34,9 +37,10 @@ class IVector:
     def z(self):
         return self.coords.get('z', None)
 
-    @property
-    def length(self):
-        return self.length()
+    # THIS DOENST WORK, would like to be able to call via .length or .length(), but havent figured it out yet
+    # @property
+    # def length(self):
+    #     return self.length()
 
     def degree(self):
         return len(self.coords)
@@ -68,6 +72,17 @@ class IVector:
         else:
             raise TypeError(f"type {other} cannot be distanced from {type(self)}")
 
+    def scaled_to_length(self, desired_length: float):
+        new_vector = self.unit()
+        if new_vector is None:
+            return Vector2(0, 0)
+
+        for ii in new_vector.coords:
+            new_vector.coords[ii] = new_vector.coords[ii] * desired_length
+
+        return new_vector
+
+
     def __eq__(self, other) -> bool:
         if not (isinstance(other, type(self)) or issubclass(type(other), type(self)) or issubclass(type(self), type(other))):
             return False
@@ -285,39 +300,75 @@ class IVector:
 
         return next(iter([x[0] for x in qualifiers if x[1] == min_dist]), None)
 
+    def as_tuple(self):
+        return tuple([v for k, v in self.coords.items()])
 
+    def angle_from(self, other=None):
+        if other is None:
+            other = Vector2(1, 0)
+
+        unit_vector_2 = self.unit().as_tuple()
+        unit_vector_1 = other.unit().as_tuple()
+        dot_product = np.dot(unit_vector_1, unit_vector_2)
+        det = np.linalg.det([unit_vector_1, unit_vector_2])
+        angle = math.atan2(-det, -dot_product) + math.pi  # atan2(y, x) or atan2(sin, cos)
+
+        return angle
 
 class Vector2 (IVector):
+    @classmethod
+    def from_ivector(cls, ivec: IVector):
+        return Vector2(ivec.x, ivec.y)
+
+    @classmethod
+    def from_vector3(cls, vec3):
+        if not type(vec3) == Vector3:
+            raise TypeError(f"input type must be {type(Vector3)} but type [{type(vec3)}] was provided")
+        return Vector2(vec3.x, vec3.y)
+
+    @classmethod
+    def from_tuple(cls, tup: Tuple[float, float]):
+        if len(tup) < 2:
+            raise ValueError(f"Can only create a Vector2 from a float tuple of size 2 or more. {tup} provided")
+
+        x = float(tup[0])
+        y = float(tup[1])
+
+        return Vector2(x, y)
+
     def __init__(self, x: float, y: float):
         IVector.__init__(self)
         self.coords['x'] = x
         self.coords['y'] = y
 
-    def as_tuple(self):
-        return (self.x, self.y)
+class Vector3(IVector):
+    @classmethod
+    def from_ivector(cls, ivec: IVector):
+        return Vector3(ivec.x, ivec.y, ivec.z)
 
-    def angle_from(self, vector = None):
-        if vector is None:
-            vector = Vector2(1, 0)
+    @classmethod
+    def from_vector2(cls, vec2: Vector2, z: float = 0):
+        if not type(vec2) == Vector2:
+            raise TypeError(f"input type must be {type(Vector2)} but type [{type(vec2)}] was provided")
 
-        if not isinstance(vector, Vector2):
-            raise TypeError(f"vector {vector} must be of type {Vector2}")
+        return Vector3(vec2.x, vec2.y, z)
 
-        vector_1 = [vector.x, vector.y]
-        vector_2 = [self.x, self.y]
+    @classmethod
+    def from_tuple(cls, tup: Tuple[float, float, float]):
+        if len(tup) < 3:
+            raise ValueError(f"Can only create a Vector3 from a float tuple of size 3 or more. {tup} provided")
 
-        dot = vector_1[0] * vector_2[0] + vector_1[1] * vector_2[1]  # dot product between [x1, y1] and [x2, y2]
-        det = vector_1[0] * vector_2[1]  - vector_1[1] * vector_2[0]  # determinant
-        angle = math.atan2(-det, -dot) + math.pi # atan2(y, x) or atan2(sin, cos)
+        x = float(tup[0])
+        y = float(tup[1])
+        z = float(tup[2])
 
-        #
-        # unit_vector_1 = vector_1 / np.linalg.norm(vector_1)
-        # unit_vector_2 = vector_2 / np.linalg.norm(vector_2)
-        # dot_product = np.dot(unit_vector_1, unit_vector_2)
-        # angle = np.arccos(dot_product)
-        #
+        return Vector3(x, y, z)
 
-        return angle
+    def __init__(self, x: float, y: float, z: float):
+        IVector.__init__(self)
+        self.coords['x'] = x
+        self.coords['y'] = y
+        self.coords['z'] = z
 
 if __name__ == "__main__":
     a = Vector2(1, 2)
